/*******************************************************************************
 * University of Illinois/NCSA
 * Open Source License
 *
 * Copyright (c) 2010 , NCSA.  All rights reserved.
 *
 * Developed by:
 * Cyberenvironments and Technologies (CET)
 * http://cet.ncsa.illinois.edu/
 *
 * National Center for Supercomputing Applications (NCSA)
 * http://www.ncsa.illinois.edu/
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal with the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimers.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in the
 *   documentation and/or other materials provided with the distribution.
 * - Neither the names of CET, University of Illinois/NCSA, nor the names
 *   of its contributors may be used to endorse or promote products
 *   derived from this Software without specific prior written permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 *******************************************************************************/
package edu.uiuc.ncsa.cet.bean;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashSet;
import java.util.UUID;


/**
 * Base class for all beans used.
 * 
 * @author Rob Kooper
 * 
 */
public class CETBean implements Serializable {
    /**
     * An interface for beans with titles. If your bean has a title property, make it implement this
     * then when the bean is fetched, if the title is missing, it will automatically be set to being
     * the same as any label.  This will end up being replaced if/when bean mappings get the ability
     * to specify default values.
     */
    public interface TitledBean{
        public String getTitle();
        public void setTitle(String title);
    }
    /** Used for serialization of object */
    private static final long serialVersionUID = 1L;

    /** Unique identifier for this bean */
    private String            uri              = null;

    public CETBean() {
        String name = getClass().getName();
        int idx = name.lastIndexOf("."); //$NON-NLS-1$
        if (idx != -1) {
            name = name.substring(idx + 1);
        }
        idx = name.lastIndexOf("Bean"); //$NON-NLS-1$
        if (idx != -1) {
            name = name.substring(0, idx);
        }
        uri = "tag:cet.ncsa.uiuc.edu,2008:/bean/" + name + "/" + UUID.randomUUID().toString(); //$NON-NLS-1$ //$NON-NLS-2$
    }

    /**
     * Return the uri of the bean.
     * 
     * @return uri of the bean
     */
    public String getUri() {
        return uri;
    }

    /**
     * Sets the uri of the bean
     * 
     * @param uri
     *            sets the uri of the bean.
     * 
     */
    public void setUri(String uri) {
        this.uri = uri;
    }
    
    Collection<String> rdfTypes; 
    /**
     * All of the rdf types associated with this bean. This is read only.
     * If a client absolutely must manage rdf types, the correct way to do this is with the bean session
     * apis, such as <code>addRDFType(Resource, Resource)</code>. 
     * @return
     */
    public Collection<String> getAllTypes(){
    	if(rdfTypes == null){
    		rdfTypes = new HashSet<String>();
    	}
    	return rdfTypes;
    }
 
    
    String label;
    public String getLabel(){
    	return label;
    }
    
    public void setLabel(String label){
    	this.label = label;
    }
}
