package edu.uiuc.ncsa.cet.bean.licensing;

/**
 * Also includes CC0 (public domain waiver).
 * 
 */
public enum CreativeCommonsLicense {
    ZERO(false, false, false, false, "zero", "http://creativecommons.org/publicdomain/zero/1.0/"),
    ATTRIBUTION(true, false, false, false, "by"),
    ATTRIBUTION_SHARE_ALIKE(true, true, false, false, "by-sa"),
    ATTRIBUTION_NO_DERIVATIVES(true, false, false, true, "by-nd"),
    ATTRIBUTION_NON_COMMERCIAL(true, false, true, false, "by-nc"),
    ATTRIBUTION_NON_COMMERCIAL_SHARE_ALIKE(true, true, true, false, "by-nc-sa"),
    ATTRIBUTION_NON_COMMERCIAL_NO_DERIVATIVES(true, false, true, true, "by-nc-nd");

    String  uri;
    String  id;
    boolean attribution;
    boolean shareAlike;
    boolean nonCommercial;
    boolean noDerivatives;

    private CreativeCommonsLicense(boolean attribution, boolean shareAlike, boolean nonCommercial, boolean noDerivatives, String id) {
        this(attribution, shareAlike, nonCommercial, noDerivatives, id, "http://creativecommons.org/licenses/" + id + "/3.0/");
    }

    private CreativeCommonsLicense(boolean attribution, boolean shareAlike, boolean nonCommercial, boolean noDerivatives, String id, String uri) {
        this.attribution = attribution;
        this.shareAlike = shareAlike;
        this.nonCommercial = nonCommercial;
        this.noDerivatives = noDerivatives;
        this.id = id;
        this.uri = uri;
    }

    public boolean isPublicDomain() {
        return this == ZERO;
    }

    public boolean isAttribution() {
        return attribution;
    }

    public boolean isShareAlike() {
        return shareAlike;
    }

    public boolean isNonCommercial() {
        return nonCommercial;
    }

    public boolean isNoDerivatives() {
        return noDerivatives;
    }

    public String getId() {
        return id;
    }

    public String getUri() {
        return uri;
    }

    public String getName() {
        if (isPublicDomain()) {
            return "Public Domain";
        }
        String s = "Attribution"; // no non-attribution licenses supported
        if (isNonCommercial()) {
            s += " Non-Commercial";
        }
        if (isNoDerivatives()) {
            s += " No Derivatives";
        }
        if (isShareAlike()) {
            s += " Share Alike";
        }
        return s;
    }
}
