/*******************************************************************************
 * University of Illinois/NCSA
 * Open Source License
 *
 * Copyright (c) 2010 , NCSA.  All rights reserved.
 *
 * Developed by:
 * Cyberenvironments and Technologies (CET)
 * http://cet.ncsa.illinois.edu/
 *
 * National Center for Supercomputing Applications (NCSA)
 * http://www.ncsa.illinois.edu/
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal with the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimers.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in the
 *   documentation and/or other materials provided with the distribution.
 * - Neither the names of CET, University of Illinois/NCSA, nor the names
 *   of its contributors may be used to endorse or promote products
 *   derived from this Software without specific prior written permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 *******************************************************************************/
/**
 * 
 */
package edu.uiuc.ncsa.cet.bean.rbac;

import java.io.Serializable;
import java.util.HashSet;

import edu.uiuc.ncsa.cet.bean.CETBean;
import edu.uiuc.ncsa.cet.bean.PersonBean;

/**
 * A group of people. Used to manage group based authorization.
 * 
 * @author Luigi Marini
 * 
 */
public class GroupBean extends CETBean implements Serializable {

	private static final long serialVersionUID = 6942002309187606913L;

	/** Pretty name of the group **/
	private String name;

	/** Members of the group **/
	private HashSet<PersonBean> members;

	/**
	 * Create an empty group.
	 */
	public GroupBean() {
		setName("");
		setMembers(new HashSet<PersonBean>());
	}

	/**
	 * Greate a group with a specific name.
	 */
	public GroupBean(String name) {
		this();
		this.setName(name);
	}

	/**
	 * Set the pretty name of the group.
	 * 
	 * @param name
	 *            the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Get the pretty name of the group.
	 * 
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * Set members of the group.
	 * 
	 * @param members
	 *            the members to set
	 */
	public void setMembers(HashSet<PersonBean> members) {
		this.members = members;
	}

	/**
	 * Get members of the group.
	 * 
	 * @return the members
	 */
	public HashSet<PersonBean> getMembers() {
		return members;
	}

	/**
	 * Add a member to the group.
	 * 
	 * @param member
	 */
	public void addMember(PersonBean member) {
		members.add(member);
	}

	/**
	 * Remove a member from a group.
	 * 
	 * @param member
	 */
	public void removeMemeber(PersonBean member) {
		members.remove(member);
	}

}
