/*******************************************************************************
 * University of Illinois/NCSA
 * Open Source License
 *
 * Copyright (c) 2010 , NCSA.  All rights reserved.
 *
 * Developed by:
 * Cyberenvironments and Technologies (CET)
 * http://cet.ncsa.illinois.edu/
 *
 * National Center for Supercomputing Applications (NCSA)
 * http://www.ncsa.illinois.edu/
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal with the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimers.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in the
 *   documentation and/or other materials provided with the distribution.
 * - Neither the names of CET, University of Illinois/NCSA, nor the names
 *   of its contributors may be used to endorse or promote products
 *   derived from this Software without specific prior written permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 *******************************************************************************/
package edu.uiuc.ncsa.cet.bean.workflow;

import java.util.Date;

import edu.uiuc.ncsa.cet.bean.CETBean;
import edu.uiuc.ncsa.cet.bean.PersonBean;

public class ViewedBean extends CETBean {
    /** Used for serialization of object */
    private static final long serialVersionUID = 1L;

    /** Date the view is created */
    private Date              date             = new Date();

    /** viewer */
    private PersonBean        viewer           = null;

    /** viewed using */
    private String            using            = null;

    /**
     * Create a new instance of the view.
     */
    public ViewedBean() {
    }

    /**
     * Return the date when the view was created.
     * 
     * @return date the view was created.
     */
    public Date getDate() {
        return date;
    }

    /**
     * Sets the date when the view was created.
     * 
     * @param date
     *            sets the date when the view was created.
     * 
     */
    public void setDate(Date date) {
        this.date = date;
    }

    /**
     * Return the PersonBean that is the viewer
     * 
     * @return PersonBean that represents the viewer
     */
    public PersonBean getViewer() {
        return viewer;
    }

    /**
     * Sets the PersonBean that represents the viewer.
     * 
     * @param creator
     *            sets the PersonBean that represents the viewer
     * 
     */
    public void setViewer(PersonBean creator) {
        this.viewer = creator;
    }

    /**
     * Return what was used to view
     * 
     * @return what was used to view
     */
    public String getUsing() {
        return using;
    }

    /**
     * Sets what was used to view
     * 
     * @param view
     *            what was used to view
     * 
     */
    public void setUsing(String using) {
        this.using = using;
    }
}
